@@ -32,23 +32,34 @@ extern "C" {
 void *
 xalloc(size_t size);
 
+struct tstring {
+  struct tstring *next;
+  unsigned length;
+  uint8_t data[1];
+};
+
+struct tstring *
+tstring_alloc (unsigned length);
+
+void
+tstring_clear(void);
+
+struct tstring *
+tstring_data(unsigned length, const char *data);
+
+struct tstring *
+tstring_hex(const char *hex);
+
+void
+tstring_print_hex(const struct tstring *s);
+
 /* Decodes a NUL-terminated hex string. */
 
-unsigned
-decode_hex_length(const char *hex);
-
-int
-decode_hex(uint8_t *dst, const char *hex);
-
-/* Allocates space */
-const uint8_t *
-decode_hex_dup(const char *hex);
-
 void
 print_hex(unsigned length, const uint8_t *data);
 
 /* The main program */
-int
+void
 test_main(void);
 
 extern int verbose;
@@ -81,71 +92,53 @@ struct nettle_mac
   hmac_##name##_update,				\
   hmac_##name##_digest,				\
 }
- 
+
+/* Test functions deallocate their inputs when finished.*/
 void
 test_cipher(const struct nettle_cipher *cipher,
-	    unsigned key_length,
-	    const uint8_t *key,
-	    unsigned length,
-	    const uint8_t *cleartext,
-	    const uint8_t *ciphertext);
+	    const struct tstring *key,
+	    const struct tstring *cleartext,
+	    const struct tstring *ciphertext);
 
 void
 test_cipher_cbc(const struct nettle_cipher *cipher,
-		unsigned key_length,
-		const uint8_t *key,
-		unsigned length,
-		const uint8_t *cleartext,
-		const uint8_t *ciphertext,
-		const uint8_t *iv);
+		const struct tstring *key,
+		const struct tstring *cleartext,
+		const struct tstring *ciphertext,
+		const struct tstring *iv);
 
 void
 test_cipher_ctr(const struct nettle_cipher *cipher,
-		unsigned key_length,
-		const uint8_t *key,
-		unsigned length,
-		const uint8_t *cleartext,
-		const uint8_t *ciphertext,
-		const uint8_t *iv);
+		const struct tstring *key,
+		const struct tstring *cleartext,
+		const struct tstring *ciphertext,
+		const struct tstring *iv);
 
 void
 test_cipher_stream(const struct nettle_cipher *cipher,
-		   unsigned key_length,
-		   const uint8_t *key,
-		   unsigned length,
-		   const uint8_t *cleartext,
-		   const uint8_t *ciphertext);
+		   const struct tstring *key,
+		   const struct tstring *cleartext,
+		   const struct tstring *ciphertext);
 
 void
 test_aead(const struct nettle_aead *aead,
-	  unsigned key_length,
-	  const uint8_t *key,
-	  unsigned auth_length,
-	  const uint8_t *authtext,
-	  unsigned length,
-	  const uint8_t *cleartext,
-	  const uint8_t *ciphertext,
-	  unsigned iv_length,
-	  const uint8_t *iv,
-	  const uint8_t *digest);
+	  const struct tstring *key,
+	  const struct tstring *authtext,
+	  const struct tstring *cleartext,
+	  const struct tstring *ciphertext,
+	  const struct tstring *iv,
+	  const struct tstring *digest);
 
 void
 test_hash(const struct nettle_hash *hash,
-	  unsigned length,
-	  const uint8_t *data,
-	  const uint8_t *digest);
+	  const struct tstring *msg,
+	  const struct tstring *digest);
 
 void
 test_hash_large(const struct nettle_hash *hash,
 		unsigned count, unsigned length,
 		uint8_t c,
-		const uint8_t *digest);
-
-void
-test_mac(const struct nettle_mac *mac,
-	 unsigned key_length, const uint8_t *key,
-	 unsigned msg_length, const uint8_t *msg,
-	 const uint8_t *digest);
+		const struct tstring *digest);
 
 void
 test_armor(const struct nettle_armor *armor,
@@ -198,29 +191,31 @@ test_dsa_key(struct dsa_public_key *pub,
 	     unsigned q_size);
 
 #endif /* WITH_HOGWEED */
+  
+/* LDATA needs to handle NUL characters. */
+#define LLENGTH(x) (sizeof(x) - 1)
+#define LDATA(x) LLENGTH(x), x
+#define LDUP(x) strlen(x), strdup(x)
+
+#define SHEX(x) ((const struct tstring *) tstring_hex(x))
+#define SDATA(x) ((const struct tstring *)tstring_data(LLENGTH(x), x))
+#define H(x) (SHEX(x)->data)
+
+#define MEMEQ(length, a, b) (!memcmp((a), (b), (length)))
+
+#define FAIL() abort()
+#define SKIP() exit(77)
+
+#define ASSERT(x) do {							\
+    if (!(x))								\
+      {									\
+	fprintf(stderr, "Assert failed %d: %s\n", __LINE__, #x);	\
+	FAIL();								\
+      }									\
+  } while(0)
 
 #ifdef __cplusplus
 }
 #endif
 
-#define H2(d, s) decode_hex((d), (s))
-#define H(x) decode_hex_dup(x)
-#define HL(x) decode_hex_length(x), decode_hex_dup(x)
-
-/* LDATA needs to handle NUL characters. */
-#define LLENGTH(x) (sizeof(x) - 1)
-#define LDATA(x) (sizeof(x) - 1), x
-#define LDUP(x) strlen(x), strdup(x)
-
-#define MEMEQ(length, a, b) (!memcmp((a), (b), (length)))
-#define MEMEQH(length, a, b) \
-((length) == decode_hex_length((b)) \
- && !memcmp((a), decode_hex_dup((b)), (length)))
-
-#define FAIL() abort()
-#define SKIP() exit(77)
-#define SUCCESS() return EXIT_SUCCESS
-
-#define ASSERT(x) do { if (!(x)) FAIL(); } while(0)
-
 #endif /* NETTLE_TESTUTILS_H_INCLUDED */
